@@ -12,6 +12,7 @@
 #include <QCoreApplication>
 #include <QDataStream>
 #include <QHash>
+#include <QStringDecoder>
 #include <QTimeZone>
 
 // TIFF 6 specs
@@ -37,7 +38,12 @@
 // EXIF 3 specs
 #define EXIF_EXIFIFD 0x8769
 #define EXIF_GPSIFD 0x8825
+#define EXIF_EXIFVERSION 0x9000
+#define EXIF_DATETIMEORIGINAL 0x9003
+#define EXIF_DATETIMEDIGITIZED 0x9004
 #define EXIF_OFFSETTIME 0x9010
+#define EXIF_OFFSETTIMEORIGINAL 0x9011
+#define EXIF_OFFSETTIMEDIGITIZED 0x9012
 #define EXIF_COLORSPACE 0xA001
 #define EXIF_PIXELXDIM 0xA002
 #define EXIF_PIXELYDIM 0xA003
@@ -47,7 +53,6 @@
 #define EXIF_LENSMODEL 0xA434
 #define EXIF_LENSSERIALNUMBER 0xA435
 #define EXIF_IMAGETITLE 0xA436
-#define EXIF_EXIFVERSION 0x9000
 
 #define EXIF_VAL_COLORSPACE_SRGB 1
 #define EXIF_VAL_COLORSPACE_UNCAL 0xFFFF
@@ -59,7 +64,8 @@
 #define GPS_LONGITUDE 4
 #define GPS_ALTITUDEREF 5
 #define GPS_ALTITUDE 6
-
+#define GPS_IMGDIRECTIONREF 16
+#define GPS_IMGDIRECTION 17
 #define EXIF_TAG_VALUE(n, byteSize) (((n) << 6) | ((byteSize) & 0x3F))
 #define EXIF_TAG_SIZEOF(dataType) (quint16(dataType) & 0x3F)
 #define EXIF_TAG_DATATYPE(dataType) (quint16(dataType) >> 6)
@@ -106,29 +112,33 @@ static const KnownTags staticTagTypes = {
     TagInfo(TIFF_IMAGEWIDTH, ExifTagType::Long),
     TagInfo(TIFF_IMAGEHEIGHT, ExifTagType::Long),
     TagInfo(TIFF_BITSPERSAMPLE, ExifTagType::Short),
-    TagInfo(TIFF_IMAGEDESCRIPTION, ExifTagType::Ascii),
-    TagInfo(TIFF_MAKE, ExifTagType::Ascii),
-    TagInfo(TIFF_MODEL, ExifTagType::Ascii),
+    TagInfo(TIFF_IMAGEDESCRIPTION, ExifTagType::Utf8),
+    TagInfo(TIFF_MAKE, ExifTagType::Utf8),
+    TagInfo(TIFF_MODEL, ExifTagType::Utf8),
     TagInfo(TIFF_ORIENT, ExifTagType::Short),
     TagInfo(TIFF_XRES, ExifTagType::Rational),
     TagInfo(TIFF_YRES, ExifTagType::Rational),
     TagInfo(TIFF_URES, ExifTagType::Short),
-    TagInfo(TIFF_SOFTWARE, ExifTagType::Ascii),
-    TagInfo(TIFF_ARTIST, ExifTagType::Ascii),
+    TagInfo(TIFF_SOFTWARE, ExifTagType::Utf8),
+    TagInfo(TIFF_ARTIST, ExifTagType::Utf8),
     TagInfo(TIFF_DATETIME, ExifTagType::Ascii),
-    TagInfo(TIFF_COPYRIGHT, ExifTagType::Ascii),
+    TagInfo(TIFF_COPYRIGHT, ExifTagType::Utf8),
     TagInfo(EXIF_EXIFIFD, ExifTagType::Long),
     TagInfo(EXIF_GPSIFD, ExifTagType::Long),
+    TagInfo(EXIF_DATETIMEORIGINAL, ExifTagType::Ascii),
+    TagInfo(EXIF_OFFSETTIMEDIGITIZED, ExifTagType::Ascii),
     TagInfo(EXIF_OFFSETTIME, ExifTagType::Ascii),
+    TagInfo(EXIF_OFFSETTIMEORIGINAL, ExifTagType::Ascii),
+    TagInfo(EXIF_OFFSETTIMEDIGITIZED, ExifTagType::Ascii),
     TagInfo(EXIF_COLORSPACE, ExifTagType::Short),
     TagInfo(EXIF_PIXELXDIM, ExifTagType::Long),
     TagInfo(EXIF_PIXELYDIM, ExifTagType::Long),
     TagInfo(EXIF_IMAGEUNIQUEID, ExifTagType::Ascii),
     TagInfo(EXIF_BODYSERIALNUMBER, ExifTagType::Ascii),
-    TagInfo(EXIF_LENSMAKE, ExifTagType::Ascii),
-    TagInfo(EXIF_LENSMODEL, ExifTagType::Ascii),
+    TagInfo(EXIF_LENSMAKE, ExifTagType::Utf8),
+    TagInfo(EXIF_LENSMODEL, ExifTagType::Utf8),
     TagInfo(EXIF_LENSSERIALNUMBER, ExifTagType::Ascii),
-    TagInfo(EXIF_IMAGETITLE, ExifTagType::Ascii),
+    TagInfo(EXIF_IMAGETITLE, ExifTagType::Utf8),
     TagInfo(EXIF_EXIFVERSION, ExifTagType::Undefined)
 };
 // clang-format on
@@ -144,7 +154,9 @@ static const KnownTags staticGpsTagTypes = {
     TagInfo(GPS_LONGITUDEREF, ExifTagType::Ascii),
     TagInfo(GPS_LONGITUDE, ExifTagType::Rational),
     TagInfo(GPS_ALTITUDEREF, ExifTagType::Byte),
-    TagInfo(GPS_ALTITUDE, ExifTagType::Rational)
+    TagInfo(GPS_ALTITUDE, ExifTagType::Rational),
+    TagInfo(GPS_IMGDIRECTIONREF, ExifTagType::Ascii),
+    TagInfo(GPS_IMGDIRECTION, ExifTagType::Rational)
 };
 // clang-format on
 
@@ -230,8 +242,8 @@ static bool checkHeader(QDataStream &ds)
 
     quint16 version;
     ds >> version;
-    if (version != 0x2A)
-        return false;
+    if (version != 0x002A && version != 0x01BC)
+        return false; // not TIFF or JXR
 
     quint32 offset;
     ds >> offset;
@@ -356,6 +368,28 @@ static void writeData(QDataStream &ds, const QVariant &value, const ExifTagType&
     }
 }
 
+static ExifTagType updateDataType(const ExifTagType &dataType, const QVariant &value, const MicroExif::Version &ver)
+{
+    if (dataType != ExifTagType::Utf8)
+        return dataType;
+
+    if (ver == MicroExif::V2)
+        return ExifTagType::Ascii;
+
+    // Note that in EXIF specs, UTF-8 is backward compatible with ASCII: all UTF-8 tags can also be ASCII.
+    // To maximize compatibility, I check if the string can be encoded in ASCII.
+    auto txt = value.toString();
+
+    // Exif ASCII data type allow only values up to 127 (7-bit ASCII).
+    auto u8 = txt.toUtf8();
+    for (auto &&c : u8) {
+        if (uchar(c) > 127)
+            return dataType;
+    }
+
+    return ExifTagType::Ascii;
+}
+
 /*!
  * \brief writeIfd
  * \param ds The stream.
@@ -364,7 +398,12 @@ static void writeData(QDataStream &ds, const QVariant &value, const ExifTagType&
  * \param knownTags List of known and supported tags.
  * \return True on success, otherwise false.
  */
-static bool writeIfd(QDataStream &ds, const MicroExif::Tags &tags, TagPos &positions, quint32 pos = 0, const KnownTags &knownTags = staticTagTypes)
+static bool writeIfd(QDataStream &ds,
+                     const MicroExif::Version &ver,
+                     const MicroExif::Tags &tags,
+                     TagPos &positions,
+                     quint32 pos = 0,
+                     const KnownTags &knownTags = staticTagTypes)
 {
     if (tags.isEmpty())
         return true;
@@ -379,7 +418,7 @@ static bool writeIfd(QDataStream &ds, const MicroExif::Tags &tags, TagPos &posit
             continue;
         }
         auto value = tags.value(key);
-        auto dataType = knownTags.value(key);
+        auto dataType = updateDataType(knownTags.value(key), value, ver);
         auto count = countBytes(dataType, value);
 
         ds << quint16(key);
@@ -401,9 +440,8 @@ static bool writeIfd(QDataStream &ds, const MicroExif::Tags &tags, TagPos &posit
         if (!knownTags.contains(key)) {
             continue;
         }
-
         auto value = tags.value(key);
-        auto dataType = knownTags.value(key);
+        auto dataType = updateDataType(knownTags.value(key), value, ver);
         auto count = countBytes(dataType, value);
         auto valueSize = count * EXIF_TAG_SIZEOF(dataType);
         if (valueSize <= 4)
@@ -523,8 +561,16 @@ static bool readIfd(QDataStream &ds, MicroExif::Tags &tags, quint32 pos = 0, con
 
         if (dataType == EXIF_TAG_DATATYPE(ExifTagType::Ascii) || dataType == EXIF_TAG_DATATYPE(ExifTagType::Utf8)) {
             auto l = readBytes(ds, count, true);
-            if (!l.isEmpty())
-                tags.insert(tagId, dataType == EXIF_TAG_DATATYPE(ExifTagType::Utf8) ? QString::fromUtf8(l) : QString::fromLatin1(l));
+            if (!l.isEmpty()) {
+                // It seems that converting to Latin 1 never detects errors so, using UTF-8.
+                // Note that if the dataType is ASCII, by EXIF specification, it must use only the
+                // first 128 values ​​so the UTF-8 conversion is correct.
+                auto dec = QStringDecoder(QStringDecoder::Utf8);
+                // QStringDecoder raise an error only after converting to QString
+                auto ut8 = QString(dec(l));
+                // If there are errors in the conversion to UTF-8, then I try with latin1 (extended ASCII)
+                tags.insert(tagId, dec.hasError() ? QString::fromLatin1(l) : ut8);
+            }
         } else if (dataType == EXIF_TAG_DATATYPE(ExifTagType::Undefined)) {
             auto l = readBytes(ds, count, false);
             if (!l.isEmpty())
@@ -852,6 +898,46 @@ void MicroExif::setDateTime(const QDateTime &dt)
     setExifString(EXIF_OFFSETTIME, timeOffset(dt.offsetFromUtc() / 60));
 }
 
+QDateTime MicroExif::dateTimeOriginal() const
+{
+    auto dt = QDateTime::fromString(exifString(EXIF_DATETIMEORIGINAL), QStringLiteral("yyyy:MM:dd HH:mm:ss"));
+    auto ofTag = exifString(EXIF_OFFSETTIMEORIGINAL);
+    if (dt.isValid() && !ofTag.isEmpty())
+        dt.setTimeZone(QTimeZone::fromSecondsAheadOfUtc(timeOffset(ofTag) * 60));
+    return(dt);
+}
+
+void MicroExif::setDateTimeOriginal(const QDateTime &dt)
+{
+    if (!dt.isValid()) {
+        m_exifTags.remove(EXIF_DATETIMEORIGINAL);
+        m_exifTags.remove(EXIF_OFFSETTIMEORIGINAL);
+        return;
+    }
+    setExifString(EXIF_DATETIMEORIGINAL, dt.toString(QStringLiteral("yyyy:MM:dd HH:mm:ss")));
+    setExifString(EXIF_OFFSETTIMEORIGINAL, timeOffset(dt.offsetFromUtc() / 60));
+}
+
+QDateTime MicroExif::dateTimeDigitized() const
+{
+    auto dt = QDateTime::fromString(exifString(EXIF_DATETIMEDIGITIZED), QStringLiteral("yyyy:MM:dd HH:mm:ss"));
+    auto ofTag = exifString(EXIF_OFFSETTIMEDIGITIZED);
+    if (dt.isValid() && !ofTag.isEmpty())
+        dt.setTimeZone(QTimeZone::fromSecondsAheadOfUtc(timeOffset(ofTag) * 60));
+    return(dt);
+}
+
+void MicroExif::setDateTimeDigitized(const QDateTime &dt)
+{
+    if (!dt.isValid()) {
+        m_exifTags.remove(EXIF_DATETIMEDIGITIZED);
+        m_exifTags.remove(EXIF_OFFSETTIMEDIGITIZED);
+        return;
+    }
+    setExifString(EXIF_DATETIMEDIGITIZED, dt.toString(QStringLiteral("yyyy:MM:dd HH:mm:ss")));
+    setExifString(EXIF_OFFSETTIMEDIGITIZED, timeOffset(dt.offsetFromUtc() / 60));
+}
+
 QString MicroExif::title() const
 {
     return exifString(EXIF_IMAGETITLE);
@@ -896,6 +982,10 @@ double MicroExif::latitude() const
 
 void MicroExif::setLatitude(double degree)
 {
+    if (qIsNaN(degree)) {
+        m_gpsTags.remove(GPS_LATITUDEREF);
+        m_gpsTags.remove(GPS_LATITUDE);
+    }
     if (degree < -90.0 || degree > 90.0)
         return; // invalid latitude
     auto adeg = qAbs(degree);
@@ -921,6 +1011,10 @@ double MicroExif::longitude() const
 
 void MicroExif::setLongitude(double degree)
 {
+    if (qIsNaN(degree)) {
+        m_gpsTags.remove(GPS_LONGITUDEREF);
+        m_gpsTags.remove(GPS_LONGITUDE);
+    }
     if (degree < -180.0 || degree > 180.0)
         return; // invalid longitude
     auto adeg = qAbs(degree);
@@ -935,17 +1029,45 @@ double MicroExif::altitude() const
     auto ref = m_gpsTags.value(GPS_ALTITUDEREF);
     if (ref.isNull())
         return qQNaN();
+    if (!m_gpsTags.contains(GPS_ALTITUDE))
+        return qQNaN();
     auto alt = m_gpsTags.value(GPS_ALTITUDE).toDouble();
     return (ref.toInt() == 0 || ref.toInt() == 2) ? alt : -alt;
 }
 
 void MicroExif::setAltitude(double meters)
 {
+    if (qIsNaN(meters)) {
+        m_gpsTags.remove(GPS_ALTITUDEREF);
+        m_gpsTags.remove(GPS_ALTITUDE);
+    }
     m_gpsTags.insert(GPS_ALTITUDEREF, quint8(meters < 0 ? 1 : 0));
     m_gpsTags.insert(GPS_ALTITUDE, meters);
 }
 
-QByteArray MicroExif::toByteArray(const QDataStream::ByteOrder &byteOrder) const
+double MicroExif::imageDirection(bool *isMagnetic) const
+{
+    auto tmp = false;
+    if (isMagnetic == nullptr)
+        isMagnetic = &tmp;
+    if (!m_gpsTags.contains(GPS_IMGDIRECTION))
+        return qQNaN();
+    auto ref = gpsString(GPS_IMGDIRECTIONREF).toUpper();
+    *isMagnetic = (ref == QStringLiteral("M"));
+    return m_gpsTags.value(GPS_IMGDIRECTION).toDouble();
+}
+
+void MicroExif::setImageDirection(double degree, bool isMagnetic)
+{
+    if (qIsNaN(degree)) {
+        m_gpsTags.remove(GPS_IMGDIRECTIONREF);
+        m_gpsTags.remove(GPS_IMGDIRECTION);
+    }
+    m_gpsTags.insert(GPS_IMGDIRECTIONREF, isMagnetic ? QStringLiteral("M") : QStringLiteral("T"));
+    m_gpsTags.insert(GPS_IMGDIRECTION, degree);
+}
+
+QByteArray MicroExif::toByteArray(const QDataStream::ByteOrder &byteOrder, const Version &version) const
 {
     QByteArray ba;
     {
@@ -956,7 +1078,51 @@ QByteArray MicroExif::toByteArray(const QDataStream::ByteOrder &byteOrder) const
     return ba;
 }
 
-bool MicroExif::write(QIODevice *device, const QDataStream::ByteOrder &byteOrder) const
+QByteArray MicroExif::exifIfdByteArray(const QDataStream::ByteOrder &byteOrder, const Version &version) const
+{
+    QByteArray ba;
+    {
+        QDataStream ds(&ba, QIODevice::WriteOnly);
+        ds.setByteOrder(byteOrder);
+        auto exifTags = m_exifTags;
+        exifTags.insert(EXIF_EXIFVERSION, version == Version::V3 ? QByteArray("0300") : QByteArray("0232"));
+        TagPos positions;
+        if (!writeIfd(ds, version, exifTags, positions))
+            return {};
+    }
+    return ba;
+}
+
+bool MicroExif::setExifIfdByteArray(const QByteArray &ba, const QDataStream::ByteOrder &byteOrder)
+{
+    QDataStream ds(ba);
+    ds.setByteOrder(byteOrder);
+    return readIfd(ds, m_exifTags, 0, staticTagTypes);
+}
+
+QByteArray MicroExif::gpsIfdByteArray(const QDataStream::ByteOrder &byteOrder, const Version &version) const
+{
+    QByteArray ba;
+    {
+        QDataStream ds(&ba, QIODevice::WriteOnly);
+        ds.setByteOrder(byteOrder);
+        auto gpsTags = m_gpsTags;
+        gpsTags.insert(GPS_GPSVERSION, QByteArray("2400"));
+        TagPos positions;
+        if (!writeIfd(ds, version, gpsTags, positions, 0, staticGpsTagTypes))
+            return {};
+        return ba;
+    }
+}
+
+bool MicroExif::setGpsIfdByteArray(const QByteArray &ba, const QDataStream::ByteOrder &byteOrder)
+{
+    QDataStream ds(ba);
+    ds.setByteOrder(byteOrder);
+    return readIfd(ds, m_gpsTags, 0, staticGpsTagTypes);
+}
+
+bool MicroExif::write(QIODevice *device, const QDataStream::ByteOrder &byteOrder, const Version &version) const
 {
     if (device == nullptr || device->isSequential() || isEmpty())
         return false;
@@ -965,14 +1131,14 @@ bool MicroExif::write(QIODevice *device, const QDataStream::ByteOrder &byteOrder
         ds.setByteOrder(byteOrder);
         if (!writeHeader(ds))
             return false;
-        if (!writeIfds(ds))
+        if (!writeIfds(ds, version))
             return false;
     }
     device->close();
     return true;
 }
 
-void MicroExif::toImageMetadata(QImage &targetImage, bool replaceExisting) const
+void MicroExif::updateImageMetadata(QImage &targetImage, bool replaceExisting) const
 {
     // set TIFF strings
     for (auto &&p : tiffStrMap) {
@@ -993,8 +1159,13 @@ void MicroExif::toImageMetadata(QImage &targetImage, bool replaceExisting) const
     }
 
     // set date and time
-    if (replaceExisting || targetImage.text(QStringLiteral(META_KEY_CREATIONDATE)).isEmpty()) {
+    if (replaceExisting || targetImage.text(QStringLiteral(META_KEY_MODIFICATIONDATE)).isEmpty()) {
         auto dt = dateTime();
+        if (dt.isValid())
+            targetImage.setText(QStringLiteral(META_KEY_MODIFICATIONDATE), dt.toString(Qt::ISODate));
+    }
+    if (replaceExisting || targetImage.text(QStringLiteral(META_KEY_CREATIONDATE)).isEmpty()) {
+        auto dt = dateTimeOriginal();
         if (dt.isValid())
             targetImage.setText(QStringLiteral(META_KEY_CREATIONDATE), dt.toString(Qt::ISODate));
     }
@@ -1015,15 +1186,47 @@ void MicroExif::toImageMetadata(QImage &targetImage, bool replaceExisting) const
         if (!qIsNaN(v))
             targetImage.setText(QStringLiteral(META_KEY_LONGITUDE), QStringLiteral("%1").arg(v, 0, 'g', 9));
     }
+    if (replaceExisting || targetImage.text(QStringLiteral(META_KEY_DIRECTION)).isEmpty()) {
+        auto v = imageDirection();
+        if (!qIsNaN(v))
+            targetImage.setText(QStringLiteral(META_KEY_DIRECTION), QStringLiteral("%1").arg(v, 0, 'g', 9));
+    }
 }
 
-MicroExif MicroExif::fromByteArray(const QByteArray &ba)
+void MicroExif::updateImageResolution(QImage &targetImage)
 {
+    if (horizontalResolution() > 0)
+        targetImage.setDotsPerMeterX(qRound(horizontalResolution() / 25.4 * 1000));
+    if (verticalResolution() > 0)
+        targetImage.setDotsPerMeterY(qRound(verticalResolution() / 25.4 * 1000));
+}
+
+MicroExif MicroExif::fromByteArray(const QByteArray &ba, bool searchHeader)
+{
+    auto ba0(ba);
+    if (searchHeader) {
+        auto idxLE = ba0.indexOf(QByteArray("II"));
+        auto idxBE = ba0.indexOf(QByteArray("MM"));
+        auto idx = -1;
+        if (idxLE > -1 && idxBE > -1)
+            idx = std::min(idxLE, idxBE);
+        else
+            idx = idxLE > -1 ? idxLE : idxBE;
+        if(idx > 0)
+            ba0 = ba0.mid(idx);
+    }
     QBuffer buf;
-    buf.setData(ba);
+    buf.setData(ba0);
     return fromDevice(&buf);
 }
 
+MicroExif MicroExif::fromRawData(const char *data, size_t size, bool searchHeader)
+{
+    if (data == nullptr || size == 0)
+        return {};
+    return fromByteArray(QByteArray::fromRawData(data, size), searchHeader);
+}
+
 MicroExif MicroExif::fromDevice(QIODevice *device)
 {
     if (device == nullptr || device->isSequential())
@@ -1088,12 +1291,18 @@ MicroExif MicroExif::fromImage(const QImage &image)
         exif.setSoftware(sw.trimmed());
     }
 
-    // TIFF Creation date and time
-    auto dt = QDateTime::fromString(image.text(QStringLiteral(META_KEY_CREATIONDATE)), Qt::ISODate);
+    // TIFF date and time
+    auto dt = QDateTime::fromString(image.text(QStringLiteral(META_KEY_MODIFICATIONDATE)), Qt::ISODate);
     if (!dt.isValid())
         dt = QDateTime::currentDateTime();
     exif.setDateTime(dt);
 
+    // EXIF original date and time
+    dt = QDateTime::fromString(image.text(QStringLiteral(META_KEY_CREATIONDATE)), Qt::ISODate);
+    if (!dt.isValid())
+        dt = QDateTime::currentDateTime();
+    exif.setDateTimeOriginal(dt);
+
     // GPS Info
     auto ok = false;
     auto alt = image.text(QStringLiteral(META_KEY_ALTITUDE)).toDouble(&ok);
@@ -1105,6 +1314,9 @@ MicroExif MicroExif::fromImage(const QImage &image)
     auto lon = image.text(QStringLiteral(META_KEY_LONGITUDE)).toDouble(&ok);
     if (ok)
         exif.setLongitude(lon);
+    auto dir = image.text(QStringLiteral(META_KEY_DIRECTION)).toDouble(&ok);
+    if (ok)
+        exif.setImageDirection(dir);
 
     return exif;
 }
@@ -1150,30 +1362,30 @@ bool MicroExif::writeHeader(QDataStream &ds) const
     return ds.status() ==  QDataStream::Ok;
 }
 
-bool MicroExif::writeIfds(QDataStream &ds) const
+bool MicroExif::writeIfds(QDataStream &ds, const Version &version) const
 {
     auto tiffTags = m_tiffTags;
     auto exifTags = m_exifTags;
     auto gpsTags = m_gpsTags;
-    updateTags(tiffTags, exifTags, gpsTags);
+    updateTags(tiffTags, exifTags, gpsTags, version);
 
     TagPos positions;
-    if (!writeIfd(ds, tiffTags, positions))
+    if (!writeIfd(ds, version, tiffTags, positions))
         return false;
-    if (!writeIfd(ds, exifTags, positions, positions.value(EXIF_EXIFIFD)))
+    if (!writeIfd(ds, version, exifTags, positions, positions.value(EXIF_EXIFIFD)))
         return false;
-    if (!writeIfd(ds, gpsTags, positions, positions.value(EXIF_GPSIFD), staticGpsTagTypes))
+    if (!writeIfd(ds, version, gpsTags, positions, positions.value(EXIF_GPSIFD), staticGpsTagTypes))
         return false;
     return true;
 }
 
-void MicroExif::updateTags(Tags &tiffTags, Tags &exifTags, Tags &gpsTags) const
+void MicroExif::updateTags(Tags &tiffTags, Tags &exifTags, Tags &gpsTags, const Version &version) const
 {
     if (exifTags.isEmpty()) {
         tiffTags.remove(EXIF_EXIFIFD);
     } else {
         tiffTags.insert(EXIF_EXIFIFD, quint32());
-        exifTags.insert(EXIF_EXIFVERSION, QByteArray("0300"));
+        exifTags.insert(EXIF_EXIFVERSION, version == Version::V3 ? QByteArray("0300") : QByteArray("0232"));
     }
     if (gpsTags.isEmpty()) {
         tiffTags.remove(EXIF_GPSIFD);
